import logging
from xmlrpc.client import Fault as SupervisorFault

from tfw.internals.networking import Scope

from .supervisor import ProcessManager, ProcessLogManager

LOG = logging.getLogger(__name__)


class ProcessHandler(ProcessManager, ProcessLogManager):
    keys = ['processmanager']
    """
    Event handler that can manage processes managed by supervisor.

    This EventHandler accepts messages that have a data['command'] key specifying
    a command to be executed.
    Every message must contain a data['process_name'] field with the name of the
    process to manage. This is the name specified in supervisor config files like so:
    [program:someprogram]

    Commands available: start, stop, restart, readlog
    (the names are as self-documenting as it gets)
    """
    def __init__(self, *, supervisor_uri, log_tail=0):
        ProcessManager.__init__(self, supervisor_uri)
        ProcessLogManager.__init__(self, supervisor_uri)
        self.log_tail = log_tail
        self.commands = {
            'start': self.start_process,
            'stop': self.stop_process,
            'restart': self.restart_process
        }

    def handle_event(self, message, connector):
        try:
            data = message['data']
            try:
                self.commands[data['command']](data['process_name'])
            except SupervisorFault as fault:
                message['data']['error'] = fault.faultString
            finally:
                message['data']['stdout'] = self.read_stdout(
                    data['process_name'],
                    self.log_tail
                )
                message['data']['stderr'] = self.read_stderr(
                    data['process_name'],
                    self.log_tail
                )
            connector.send_message(message, scope=Scope.WEBSOCKET)
        except KeyError:
            LOG.error('IGNORING MESSAGE: Invalid message received: %s', message)
